window.Gleam.initSearch([{"doc":"fmt","title":"fmt","content":"# fmt\n\nType-safe (string) formatting for Gleam.\n\nHeavily inspired by [&quot;Type-safe functional formatted IO&quot;](https://okmij.org/ftp/typed-formatting/index.html) by Oleg Kiselyov.\n\n## Usage\n\nThe following code would print out `Movie: Inside Out; Rating: 5; Price: $24.99`:\n\n```gleam\nimport fmt\nimport gleam/io\n\n// This will print out &quot;Movie: Inside Out; Rating: 5; Price: $24.99&quot;\npub fn main() {\n  fmt.lit(&quot;Movie: &quot;)\n  |&gt; fmt.cat(fmt.string())\n  |&gt; fmt.cat(fmt.lit(&quot;; Rating: &quot;))\n  |&gt; fmt.cat(fmt.int())\n  |&gt; fmt.cat(fmt.lit(&quot;; Price: $&quot;))\n  |&gt; fmt.cat(fmt.float())\n  |&gt; fmt.sprintf3(&quot;Inside Out&quot;, 5, 24.99)\n  |&gt; io.println\n}\n```\n\nIn this example, \n\n- `lit` is used to specify literal string values.\n- `string` is used to specify a `String` placeholder.\n- `int` is used to specify an `Int` placeholder.\n- `float` is used to specify a `Float` placeholder.\n- `cat` is used to concatenate the formatting specifications.\n- `sprintf3` is used to generate the string from the given specification.\n\nFor more details and examples, see the documentation and the tests.\n\n### Type Safety\n\nIt is **type-safe**: the compiler will ensure that you have set everything up correctly, both in terms of the number of expected arguments and their types.\n\nCheck out this example:\n\n```gleam\npub fn will_fail() {\n  fmt.lit(&quot;Name: &quot;)\n  |&gt; fmt.cat(fmt.string())\n  |&gt; fmt.cat(fmt.lit(&quot;; Age: &quot;))\n  |&gt; fmt.cat(fmt.int())\n  |&gt; fmt.sprintf2(&quot;Juan&quot;, &quot;Carlos&quot;)\n}\n```\n\nIf you were to check this with `gleam check`, you would get an error something like this:\n\n```\nerror: Type mismatch\n   ┌─ ./fmt/test/fmt_test.gleam:82:27\n   │\n82 │   |&gt; fmt.sprintf2(&quot;Juan&quot;, &quot;Carlos&quot;)\n   │                           ^^^^^^^^\n\nExpected type:\n\n    Int\n\nFound type:\n\n    String\n```\n\n## License\n\n[![license MIT or Apache\n2.0](https://img.shields.io/badge/license-MIT%20or%20Apache%202.0-blue)](https://github.com/mooreryan/gleam_qcheck)\n\nCopyright (c) 2024 Ryan M. Moore\n\nLicensed under the Apache License, Version 2.0 or the MIT license, at your option. This program may not be copied, modified, or distributed except according to those terms.\n\n","url":"index.html"},{"doc":"fmt","title":"Fmt","content":"pub opaque type Fmt(a, b)\n `Fmt` is an opaque type that represents a formatting specification.\n\nSynonyms:\nfmt.Fmt\nfmt Fmt","url":"fmt.html#Fmt"},{"doc":"fmt","title":"a","content":"pub fn a(show show: fn(a) -&gt; String) -&gt; Fmt(b, fn(a) -&gt; b)\n `a(show)` returns a format specification for a value of type `a` \n given a function `show` that converts values of type `a` to values of type\n `String`.\n \n This is useful for formatting custom types or other compound data types.\n \n Here is an example of formatting an integer list (`List(Int)`):\n \n \n import fmt\n import gleam/int\n import gleam/list\n import gleam/string\n import gleeunit/should\n \n pub fn formatting_int_list_test() {\n   let show_list = fn(lst) {\n     lst\n     |&gt; list.map(int.to_string)\n     |&gt; string.join(with: &quot;; &quot;)\n   }\n\n   fmt.lit(&quot;[&quot;)\n   |&gt; fmt.cat(fmt.a(show_list))\n   |&gt; fmt.cat(fmt.lit(&quot;]&quot;))\n   |&gt; fmt.sprintf1([1, 2, 3])\n   |&gt; should.equal(&quot;[1; 2; 3]&quot;)\n }\n \n \n Here is an example of formatting a custom data type:\n \n \n import fmt\n import gleam/float\n import gleam/int\n import gleam/list\n import gleam/string\n import gleeunit/should\n \n type Either(a, b) {\n   First(a)\n   Second(b)\n }\n \n fn either_to_string(\n   either: Either(a, b),\n   a_to_string: fn(a) -&gt; String,\n   b_to_string: fn(b) -&gt; String,\n ) -&gt; String {\n   case either {\n     First(a) -&gt; &quot;First: &quot; &lt;&gt; a_to_string(a)\n     Second(b) -&gt; &quot;Second: &quot; &lt;&gt; b_to_string(b)\n   }\n }\n \n pub fn formatting_custom_types_test() {\n   fmt.lit(&quot;Which is it? &quot;)\n   |&gt; fmt.cat(fmt.a(show: either_to_string(_, int.to_string, float.to_string)))\n   |&gt; fmt.sprintf1(Second(2.71828))\n   |&gt; should.equal(&quot;Which is it? Second: 2.71828&quot;)\n }\n \n \nSynonyms:\nfmt.a\nfmt a","url":"fmt.html#a"},{"doc":"fmt","title":"bool","content":"pub fn bool() -&gt; Fmt(a, fn(Bool) -&gt; a)\n `bool()` returns a format specification for a boolean value.  You could\n think of this as a placeholder that must be filled with a value of the type\n `Bool`.\n \nSynonyms:\nfmt.bool\nfmt bool","url":"fmt.html#bool"},{"doc":"fmt","title":"cat","content":"pub fn cat(\n  fmt_left: Fmt(a, b),\n  fmt_right: Fmt(c, a),\n) -&gt; Fmt(c, b)\n `cat(fmt_left, fmt_right)` concatenates the two formatting specifications.  \n This is the way to combine little formatting specifications into one big \n formatting specification.  You would normally compose these with the pipe \n `(|&gt;)`.\n \n \n import fmt\n import gleeunit/should\n \n pub fn cat_test() {\n   fmt.lit(&quot;Name: &quot;)\n   |&gt; fmt.cat(fmt.string())\n   |&gt; fmt.cat(fmt.lit(&quot;; Age: &quot;))\n   |&gt; fmt.cat(fmt.int())\n   |&gt; fmt.sprintf2(&quot;Juan&quot;, 45)\n   |&gt; should.equal(&quot;Name: Juan; Age: 45&quot;)\n }\n \n \nSynonyms:\nfmt.cat\nfmt cat","url":"fmt.html#cat"},{"doc":"fmt","title":"float","content":"pub fn float() -&gt; Fmt(a, fn(Float) -&gt; a)\n `float()` returns a format specification for a floating-point value.  You\n could think of this as a placeholder that must be filled with a value of the\n type `Float`.\n \nSynonyms:\nfmt.float\nfmt float","url":"fmt.html#float"},{"doc":"fmt","title":"int","content":"pub fn int() -&gt; Fmt(a, fn(Int) -&gt; a)\n `int()` returns a format specification for an integer value.  You could\n think of this as a placeholder that must be filled with a value of the type\n `Int`.\n \nSynonyms:\nfmt.int\nfmt int","url":"fmt.html#int"},{"doc":"fmt","title":"lit","content":"pub fn lit(str: String) -&gt; Fmt(a, a)\n `lit(str)` returns a format specification for the literal string value \n provided by the argument `str`.\n \n \n import fmt\n import gleeunit/should\n \n pub fn lit_test() {\n   fmt.lit(&quot;Hello, Gleam!&quot;)\n   |&gt; fmt.sprintf\n   |&gt; should.equal(&quot;Hello, Gleam!&quot;)\n }\n \n \nSynonyms:\nfmt.lit\nfmt lit","url":"fmt.html#lit"},{"doc":"fmt","title":"sprintf","content":"pub fn sprintf(fmt: Fmt(String, a)) -&gt; a\n `sprintf(fmt)` takes a formatting specification and returns a curried \n function for generating the specified string.\n \n You likely use the `sprintfN` variants of this function rather than this \n one, as those eliminate the need to work with the curried functions.\n \n Here is an example. I have annotated some of the types, which may clarify \n some things.\n \n \n pub fn sprintf_test() {\n   let fmt: fmt.Fmt(String, fn(String) -&gt; fn(Int) -&gt; fn(Bool) -&gt; String) =\n     fmt.lit(&quot;Name: &quot;)\n     |&gt; fmt.cat(fmt.string())\n     |&gt; fmt.cat(fmt.lit(&quot;; Age: &quot;))\n     |&gt; fmt.cat(fmt.int())\n     |&gt; fmt.cat(fmt.lit(&quot;; Acct. Active: &quot;))\n     |&gt; fmt.cat(fmt.bool())\n\n   let f: fn(String) -&gt; fn(Int) -&gt; fn(Bool) -&gt; String = fmt.sprintf(fmt)\n\n   f(&quot;Juan&quot;)(45)(True)\n   |&gt; should.equal(&quot;Name: Juan; Age: 45; Acct. Active: True&quot;)\n \n \n Alternatively, you may rather use the `uncurry` function with the matching arity so that you can continue the pipeline.\n \n \n pub fn sprintf_uncurry_test() {\n   fmt.lit(&quot;Name: &quot;)\n   |&gt; fmt.cat(fmt.string())\n   |&gt; fmt.cat(fmt.lit(&quot;; Age: &quot;))\n   |&gt; fmt.cat(fmt.int())\n   |&gt; fmt.cat(fmt.lit(&quot;; Acct. Active: &quot;))\n   |&gt; fmt.cat(fmt.bool())\n   |&gt; fmt.sprintf\n   |&gt; fmt.uncurry3(&quot;Juan&quot;, 45, True)\n   |&gt; should.equal(&quot;Name: Juan; Age: 45; Acct. Active: True&quot;)\n }\n \nSynonyms:\nfmt.sprintf\nfmt sprintf","url":"fmt.html#sprintf"},{"doc":"fmt","title":"sprintf1","content":"pub fn sprintf1(fmt: Fmt(String, fn(a) -&gt; b), x1: a) -&gt; b\n `sprintf1(fmt, x1)` takes a formatting specification `fmt` and a value `x1`, \n and returns the specified string with the appropriate placeholders filled by\n the given values.\n \nSynonyms:\nfmt.sprintf1\nfmt sprintf1","url":"fmt.html#sprintf1"},{"doc":"fmt","title":"sprintf2","content":"pub fn sprintf2(\n  fmt: Fmt(String, fn(a) -&gt; fn(b) -&gt; c),\n  x1: a,\n  x2: b,\n) -&gt; c\n `sprintf2(fmt, x1, x2)` takes a formatting specification `fmt` and two\n values `x1` and `x2`, and returns the specified string with the appropriate\n placeholders filled by the given values.\n \nSynonyms:\nfmt.sprintf2\nfmt sprintf2","url":"fmt.html#sprintf2"},{"doc":"fmt","title":"sprintf3","content":"pub fn sprintf3(\n  fmt: Fmt(String, fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; d),\n  x1: a,\n  x2: b,\n  x3: c,\n) -&gt; d\n `sprintf3(fmt, x1, x2, x3)` takes a formatting specification `fmt` and three\n values `x1`, `x2`, and `x3`, and returns the specified string with the\n appropriate placeholders filled by the given values.\n \nSynonyms:\nfmt.sprintf3\nfmt sprintf3","url":"fmt.html#sprintf3"},{"doc":"fmt","title":"sprintf4","content":"pub fn sprintf4(\n  fmt: Fmt(String, fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; e),\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n) -&gt; e\n `sprintf4(fmt, x1, x2, x3, x4)` takes a formatting specification `fmt` and\n four values `x1`, `x2`, `x3`, and `x4`, and returns the specified string\n with the appropriate placeholders filled by the given values.\n \nSynonyms:\nfmt.sprintf4\nfmt sprintf4","url":"fmt.html#sprintf4"},{"doc":"fmt","title":"sprintf5","content":"pub fn sprintf5(\n  fmt: Fmt(String, fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; f),\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n  x5: e,\n) -&gt; f\n `sprintf5(fmt, x1, x2, x3, x4, x5)` takes a formatting specification `fmt`\n and five values `x1`, `x2`, `x3`, `x4`, and `x5`, and returns the specified\n string with the appropriate placeholders filled by the given values.\nSynonyms:\nfmt.sprintf5\nfmt sprintf5","url":"fmt.html#sprintf5"},{"doc":"fmt","title":"sprintf6","content":"pub fn sprintf6(\n  fmt: Fmt(\n    String,\n    fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; g,\n  ),\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n  x5: e,\n  x6: f,\n) -&gt; g\n `sprintf6(fmt, x1, x2, x3, x4, x5, x6)` takes a formatting specification\n `fmt` and six values `x1`, `x2`, `x3`, `x4`, `x5`, and `x6`, and returns\n the specified string with the appropriate placeholders filled by the given\n values.\n \nSynonyms:\nfmt.sprintf6\nfmt sprintf6","url":"fmt.html#sprintf6"},{"doc":"fmt","title":"sprintf7","content":"pub fn sprintf7(\n  fmt: Fmt(\n    String,\n    fn(a) -&gt;\n      fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; fn(g) -&gt; h,\n  ),\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n  x5: e,\n  x6: f,\n  x7: g,\n) -&gt; h\n `sprintf7(fmt, x1, x2, x3, x4, x5, x6, x7)` takes a formatting specification\n `fmt` and seven values `x1`, `x2`, `x3`, `x4`, `x5`, `x6`, and `x7`, and\n returns the specified string with the appropriate placeholders filled by the\n given values.\n \nSynonyms:\nfmt.sprintf7\nfmt sprintf7","url":"fmt.html#sprintf7"},{"doc":"fmt","title":"sprintf8","content":"pub fn sprintf8(\n  fmt: Fmt(\n    String,\n    fn(a) -&gt;\n      fn(b) -&gt;\n        fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; fn(g) -&gt; fn(h) -&gt; i,\n  ),\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n  x5: e,\n  x6: f,\n  x7: g,\n  x8: h,\n) -&gt; i\n `sprintf8(fmt, x1, x2, x3, x4, x5, x6, x7, x8)` takes a formatting\n specification `fmt` and eight values `x1`, `x2`, `x3`, `x4`, `x5`, `x6`,\n `x7`, and `x8`, and returns the specified string with the appropriate\n placeholders filled by the given values.\n \nSynonyms:\nfmt.sprintf8\nfmt sprintf8","url":"fmt.html#sprintf8"},{"doc":"fmt","title":"sprintf9","content":"pub fn sprintf9(\n  fmt: Fmt(\n    String,\n    fn(a) -&gt;\n      fn(b) -&gt;\n        fn(c) -&gt;\n          fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; fn(g) -&gt; fn(h) -&gt; fn(i) -&gt; j,\n  ),\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n  x5: e,\n  x6: f,\n  x7: g,\n  x8: h,\n  x9: i,\n) -&gt; j\n `sprintf9(fmt, x1, x2, x3, x4, x5, x6, x7, x8, x9)` takes a formatting\n specification `fmt` and nine values `x1`, `x2`, `x3`, `x4`, `x5`, `x6`,\n `x7`, `x8`, and `x9`, and returns the specified string with the appropriate\n placeholders filled by the given values.\n \nSynonyms:\nfmt.sprintf9\nfmt sprintf9","url":"fmt.html#sprintf9"},{"doc":"fmt","title":"string","content":"pub fn string() -&gt; Fmt(a, fn(String) -&gt; a)\n `string()` returns a format specification for a string value.  You could\n think of this as a placeholder that must be filled with a value of the type\n `String`.\n \nSynonyms:\nfmt.string\nfmt string","url":"fmt.html#string"},{"doc":"fmt","title":"uncurry2","content":"pub fn uncurry2(f: fn(a) -&gt; fn(b) -&gt; c, x1: a, x2: b) -&gt; c\n `uncurry2(f, x1, x2)` takes a curried function `f` and two arguments `x1`\n and `x2`, and returns the result of applying the function `f` to the \n arguments.\n \n This is useful for continuing a pipeline when using the curried version of \n `sprintf`.\n \nSynonyms:\nfmt.uncurry2\nfmt uncurry2","url":"fmt.html#uncurry2"},{"doc":"fmt","title":"uncurry3","content":"pub fn uncurry3(\n  f: fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; d,\n  x1: a,\n  x2: b,\n  x3: c,\n) -&gt; d\n `uncurry3(f, x1, x2, x3)` takes a curried function `f` and three arguments\n `x1`, `x2`, and `x3`, and returns the result of applying the function `f` to\n the arguments.\n \n This is useful for continuing a pipeline when using the curried version of\n `sprintf`.\nSynonyms:\nfmt.uncurry3\nfmt uncurry3","url":"fmt.html#uncurry3"},{"doc":"fmt","title":"uncurry4","content":"pub fn uncurry4(\n  f: fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; e,\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n) -&gt; e\n `uncurry4(f, x1, x2, x3, x4)` takes a curried function `f` and four\n arguments `x1`, `x2`, `x3`, and `x4`, and returns the result of applying\n the function `f` to the arguments.\n \n This is useful for continuing a pipeline when using the curried version of\n `sprintf`.\n \nSynonyms:\nfmt.uncurry4\nfmt uncurry4","url":"fmt.html#uncurry4"},{"doc":"fmt","title":"uncurry5","content":"pub fn uncurry5(\n  f: fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; f,\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n  x5: e,\n) -&gt; f\n `uncurry5(f, x1, x2, x3, x4, x5)` takes a curried function `f` and five\n arguments `x1`, `x2`, `x3`, `x4`, and `x5`, and returns the result of\n applying the function `f` to the arguments.\n \n This is useful for continuing a pipeline when using the curried version of\n `sprintf`.\n \nSynonyms:\nfmt.uncurry5\nfmt uncurry5","url":"fmt.html#uncurry5"},{"doc":"fmt","title":"uncurry6","content":"pub fn uncurry6(\n  f: fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; g,\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n  x5: e,\n  x6: f,\n) -&gt; g\n `uncurry6(f, x1, x2, x3, x4, x5, x6)` takes a curried function `f` and six\n arguments `x1`, `x2`, `x3`, `x4`, `x5`, and `x6`, and returns the result of\n applying the function `f` to the arguments.\n \n This is useful for continuing a pipeline when using the curried version of\n `sprintf`.\n \nSynonyms:\nfmt.uncurry6\nfmt uncurry6","url":"fmt.html#uncurry6"},{"doc":"fmt","title":"uncurry7","content":"pub fn uncurry7(\n  f: fn(a) -&gt;\n    fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; fn(g) -&gt; h,\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n  x5: e,\n  x6: f,\n  x7: g,\n) -&gt; h\n `uncurry7(f, x1, x2, x3, x4, x5, x6, x7)` takes a curried function `f` and\n seven arguments `x1`, `x2`, `x3`, `x4`, `x5`, `x6`, and `x7`, and returns\n the result of applying the function `f` to the arguments.\n \n This is useful for continuing a pipeline when using the curried version of\n `sprintf`.\n \nSynonyms:\nfmt.uncurry7\nfmt uncurry7","url":"fmt.html#uncurry7"},{"doc":"fmt","title":"uncurry8","content":"pub fn uncurry8(\n  f: fn(a) -&gt;\n    fn(b) -&gt;\n      fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; fn(g) -&gt; fn(h) -&gt; i,\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n  x5: e,\n  x6: f,\n  x7: g,\n  x8: h,\n) -&gt; i\n `uncurry8(f, x1, x2, x3, x4, x5, x6, x7, x8)` takes a curried function `f`\n and eight arguments `x1`, `x2`, `x3`, `x4`, `x5`, `x6`, `x7`, and `x8`, and\n returns the result of applying the function `f` to the arguments.\n \n This is useful for continuing a pipeline when using the curried version of\n `sprintf`.\n \nSynonyms:\nfmt.uncurry8\nfmt uncurry8","url":"fmt.html#uncurry8"},{"doc":"fmt","title":"uncurry9","content":"pub fn uncurry9(\n  f: fn(a) -&gt;\n    fn(b) -&gt;\n      fn(c) -&gt;\n        fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; fn(g) -&gt; fn(h) -&gt; fn(i) -&gt; j,\n  x1: a,\n  x2: b,\n  x3: c,\n  x4: d,\n  x5: e,\n  x6: f,\n  x7: g,\n  x8: h,\n  x9: i,\n) -&gt; j\n `uncurry9(f, x1, x2, x3, x4, x5, x6, x7, x8, x9)` takes a curried function\n `f` and nine arguments `x1`, `x2`, `x3`, `x4`, `x5`, `x6`, `x7`, `x8`, and\n `x9`, and returns the result of applying the function `f` to the arguments.\n \n This is useful for continuing a pipeline when using the curried version of\n `sprintf`.\n \nSynonyms:\nfmt.uncurry9\nfmt uncurry9","url":"fmt.html#uncurry9"},{"doc":"fmt","title":"fmt","content":" This module provides functions for creating formatting specifications and \n generating strings from those specificaitons in a type-safe way.\n \n ## Overview\n\n - Functions to build basic formatting specifications\n     - `lit`: for literal strings\n     - `bool`: for `Bool` values\n     - `float`: for `Float` values\n     - `int`: for `Int` values\n     - `string`: for `String` values\n     - `a`: for custom types and compound data types\n - The `cat` function is used to concatenate formatting specifications.\n - The `sprintf` function, and its variants `sprintf1`, `sprintf2`, etc., are\n used to generate strings from formatting specifications.\n \n For additional information, check out the examples provided in this \n module&#39;s documentation, as well as those in the README and the tests, which\n can be found in the project repository.\n \n *Note: The examples in this file use functions from the gleeunit package \n for illustration.*","url":"fmt.html"}]);